package com.aurorasoftworks.signal.examples.ui.mvc.lwuit;



import com.aurorasoftworks.signal.examples.core.IAccount;
import com.sun.lwuit.Command;
import com.sun.lwuit.Dialog;
import com.sun.lwuit.Form;
import com.sun.lwuit.Label;
import com.sun.lwuit.TextField;
import com.sun.lwuit.events.DataChangedListener;
import com.sun.lwuit.layouts.BoxLayout;


public class AccountInfoView extends Form
{
	public static final Command NEXT_CMD = new Command("Next");
	
	public static final Command CANCEL_CMD = new Command("Cancel");
	
	public static final Command [] VALIDATION_DIALOG_CMDS
			= {new Command("OK")};

	private TextField firstNameText = new TextField("");
	
	private TextField lastNameText = new TextField("");
	
	private IAccount account;

	public AccountInfoView()
	{
		setLayout(new BoxLayout(BoxLayout.Y_AXIS));
		
		addComponent(new Label("First name"));
		addComponent(firstNameText);
		addComponent(new Label("Last name"));
		addComponent(lastNameText);
		
		firstNameText.addDataChangeListener(new DataChangedListener()
		{	
			public void dataChanged(int type, int index)
			{
				account.setFirstName(firstNameText.getText());
			}
		});
		
		lastNameText.addDataChangeListener(new DataChangedListener()
		{	
			public void dataChanged(int type, int index)
			{
				account.setLastName(lastNameText.getText());
			}
		});
		
		addCommand(CANCEL_CMD);
		addCommand(NEXT_CMD);
	}

	public void setAccount(IAccount account)
	{
		this.account = account;
		firstNameText.setText(account.getFirstName());
		lastNameText.setText(account.getLastName());
	}

	public IAccount getAccount()
	{
		return account;
	}
	
	protected boolean isValid(TextField text)
	{
		return text.getText().length() > 0;
	}
	
	public boolean validate()
	{
		boolean result = isValid(firstNameText) && isValid(lastNameText);
		
		if (!result)
		{
			Dialog.show("Validation error",
					"First and last name need to be provided",
					VALIDATION_DIALOG_CMDS,
					Dialog.TYPE_WARNING, null, 0);
		}
		
		return result;
	}
}
