package com.aurorasoftworks.signal.examples.ui.mvc.midp;


import javax.microedition.lcdui.Command;

import com.aurorasoftworks.signal.examples.core.IAccount;
import com.aurorasoftworks.signal.examples.core.IAccountService;
import com.aurorasoftworks.signal.runtime.ui.mvc.IDispatcher;
import com.aurorasoftworks.signal.runtime.ui.mvc.IDispatcherAware;
import com.aurorasoftworks.signal.runtime.ui.mvc.midp.AbstractViewController;
import com.aurorasoftworks.signal.runtime.ui.mvc.midp.ICommandHandler;



public class AccountListViewController extends AbstractViewController
		implements IAccountCreationEvent, IAccountListRequestedEvent,
		IDispatcherAware
{
	private IAccountService accountService;
	
	private INewAccountEvent newAccountEvent;
	
	private IEditAccountEvent editAccountEvent;
	
	private IDispatcher dispatcher;
	
	public AccountListViewController(AccountListView view,
			IAccountService accountService)
	{
		super(view);
		this.accountService = accountService;
		
		registerCommandHandler(AccountListView.NEW_ACCOUNT_CMD,
				new ICommandHandler()
		{
			public void handleCommand(Command cmd)
			{
				newAccountEvent.onNewAccount();
			}
		});
		registerCommandHandler(AccountListView.EDIT_ACCOUNT_CMD,
				new ICommandHandler()
		{
			public void handleCommand(Command cmd)
			{
				editAccountEvent.onEditAccount(
						getAccountList().getSelectedAccount());
			}
		});
		registerCommandHandler(AccountListView.EXIT_CMD, new ICommandHandler()
		{
			public void handleCommand(Command cmd)
			{
				dispatcher.exit();		
			}
		});
	}
	
	protected AccountListView getAccountList()
	{
		return (AccountListView) getView();
	}
	
	public void onAccountListRequested()
	{
		refreshAccountList();		
	}

	protected void refreshAccountList()
	{
		getAccountList().setAccounts(accountService.getAccounts());
	}

	public IDispatcher getDispatcher()
	{
		return dispatcher;
	}

	public void setDispatcher(IDispatcher dispatcher)
	{
		this.dispatcher = dispatcher;
	}

	public INewAccountEvent getNewAccountEvent()
	{
		return newAccountEvent;
	}

	public void setNewAccountEvent(INewAccountEvent newAccountEvent)
	{
		this.newAccountEvent = newAccountEvent;
	}

	public IEditAccountEvent getEditAccountEvent()
	{
		return editAccountEvent;
	}

	public void setEditAccountEvent(IEditAccountEvent editAccountEvent)
	{
		this.editAccountEvent = editAccountEvent;
	}

	public void onAccountCancelled(IAccount account)
	{
		/* No-op */
	}

	public void onAccountSubmitted(IAccount account)
	{
		refreshAccountList();	
	}
}
