/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.runtime.core.context;


/**
 * A common superclass for implementations of {@link IBeanReference}.
 * 
 * @author Marek
 */
public abstract class AbstractBeanReference implements IBeanReference
{
	/**
	 * Name of the wrapped bean.
	 */
	private String name;
	
	/**
	 * Wrapped bean.
	 */
	private Object bean;
	
	/**
	 * Creates a new instance of AbstractBeanReference.
	 * The wrapped bean must be later specified by invoking
	 * {@link #setBean(Object)}. 
	 * 
	 * @param name name of the bean to wrapped
	 */
	protected AbstractBeanReference(String name)
	{
		this.name = name;
	}
	
	/**
	 * Creates a new instance of AbstractBeanReference.
	 * 
	 * @param name name of the bean to wrapped
	 * @param bean the bean to be wrapped
	 */
	protected AbstractBeanReference(String name, Object bean)
	{
		this(name);
		this.bean = bean;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getBean() throws ContextException
	{
		return bean;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Class getBeanClass()
	{
		return bean.getClass();
	}

	/**
	 * Allows subclasses to set the wrapped bean. Useful for
	 * deferred instantiation.
	 * 
	 * @param bean
	 * @throws ContextException
	 */
	protected void setBean(Object bean) throws ContextException
	{
		this.bean = bean;
	}
}
