/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.runtime.core.context;

import java.util.Vector;

import com.aurorasoftworks.signal.runtime.core.context.proxy.IProxyFactory;



/**
 * An IoC container. This is the central part of the Signal Framework
 * API. The container maintains a map of beans (named objects)
 * instantiated by injecting the dependencies they declared in an
 * external configuration source.
 * 
 * This interface was originally coded to support Spring context
 * instantiation on the Java ME platform. Is is, however, designed to be generic
 * and not to depend on any J2ME or Spring APIs so that other types of contexts
 * and runtime environments can be supported if needed.
 * 
 * The container is mutable and new beans can therefore
 * be registered after instantiation.
 * 
 * All implementations of this interface are expected to be thread-safe.
 * 
 * @author Marek
 */
public interface IContext
{
	/**
	 * Retrieves a bean previously registered with the specified name.
	 * 
	 * @param name name of a bean to be retrieved
	 * @return a bean with the provided name or <tt>null</tt> if one
	 *         does not exist
	 */
	Object getBean(String name) throws ContextException;
	
	/**
	 * Returns names of the beans of the specified type
	 * @param beanClass
	 * @return
	 */
	Vector getBeanNames(Class beanClass) throws ContextException;
	
	Vector getBeans(Class beanClass) throws ContextException;
	
	Object getBean(Class beanClass) throws ContextException;
	
	/**
	 * Registers a bean using the provided name. If the provided name
	 * is already in use, the previous bean is replaced with the new one
	 * in this context. Note that this does not affect injected references
	 * to the previous bean.
	 * 
	 * @param name   a name of the bean being registered
	 * @param object a bean to be registered
	 */
	void registerBean(String name, Object object) throws ContextException;
	
	/**
	 * Determines whether a bean with the specified name already
	 * exists.
	 * 
	 * @param  name a name of a bean to be located in the context 
	 * @return <tt>true</tt> if a bean with the provided name exists,
	 *         <tt>false</tt> otherwise
	 */
	boolean containsBean(String name);
	
	IProxyFactory getProxyFactory();
}
