package com.aurorasoftworks.signal.runtime.ui.mvc;


import java.util.Stack;

import com.aurorasoftworks.signal.runtime.core.context.IContextAware;
import com.aurorasoftworks.signal.runtime.core.context.proxy.IMethodHandler;


public abstract class AbstractDispatcher
		extends AbstractControllerMethodInterceptor
		implements IContextAware
{
	private Stack flowCallers = new Stack();
	
	private boolean endFlow;
	
	protected IController currentController;

	protected synchronized Object onControllerMethodInvoked(
			IController controller, IMethodHandler handler,
			Object[] args) throws Exception
	{		
		if (controller != currentController)
		{
			if (currentController != null)
			{
				deactivateController(currentController);
			}

			checkFlow(controller);
			
			checkControllerState(controller);

			currentController = controller;
			activateController(currentController);
		}
		else
		{
			checkFlow(controller);
		}
		
		return handler.invoke(controller, args);
	}
	
	protected synchronized Object onFlowCallerMethodInvoked(
			IController controller,
			IMethodHandler handler, Object[] args) throws Exception
	{
		endFlow = true;
		return handler.invoke(controller, args);
	}
	
	protected void checkControllerState(IController controller)
	{
		checkDispatcherAware(controller);
	}
	
	protected abstract void checkDispatcherAware(IController controller);
	
	protected void checkFlow(IController newController)
			throws Exception
	{
		if (endFlow)
		{
			endFlow = false;
			endFlow(newController);
		}
		else if (newController instanceof IFlowController)
		{
			beginFlow((IFlowController) newController);
		}
	}

	protected void beginFlow(IFlowController newController)
			throws Exception
	{
		if (currentController == null)
		{
			throw new RuntimeException("Cannot start a flow: "
					+ "there is no active controller");
		}
		
		flowCallers.push(currentController);

		((IFlowController) newController).setCaller(
				createFlowCallerProxy(currentController));
	}
	
	protected void endFlow(IController newController) throws Exception
	{
		flowCallers.pop();
		
		if (!flowCallers.isEmpty())
		{
			((IFlowController) newController).setCaller(createFlowCallerProxy(
					(IController) flowCallers.peek()));
		}
	}
	
	protected IController createFlowCallerProxy(IController controller)
			throws Exception
	{
		return (IController) getProxyFactory().createProxy(
				/* Get the original proxy for the current controller */
				getControllerProxy(controller),
				getFlowCallerHandlers());
	}
	
	protected abstract void activateController(IController controller);

	protected abstract void deactivateController(IController controller);

	protected IController getCurrentController()
	{
		return currentController;
	}
	
	protected int getFlowLevel()
	{
		return flowCallers.size();
	}
}
