/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.runtime.ui.mvc;


import com.aurorasoftworks.signal.runtime.core.context.proxy.IProxyTarget;


/**
 * A super-interface for all MVC controllers. Every controller
 * needs to support AOP proxies and therefore needs to implement
 * {@link IProxyTarget}.
 * 
 * This interface can be implemented by controllers of various levels
 * of granularity: wizard/flow controllers and view controllers. 
 * 
 * @author Marek
 */
public interface IController extends IProxyTarget
{
	/**
	 * Invoked by the framework to indicate that this controller
	 * becomes the active one.
	 */
	void activate();
	
	/**
	 * Invoked by the framework to indicate that this controller
	 * ceases to be the active one.
	 */
	void deactivate();
}
