/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.runtime.core.context.proxy;


/**
 * A common super-class for implementations of {@link IProxy}.
 * 
 * @author Marek
 */
public abstract class AbstractProxy implements IProxy
{
	protected IProxyTarget target;

	protected IInvocationHandler handler;

	void setTarget(IProxyTarget target)
	{
		this.target = target;
	}

	void setHandler(IInvocationHandler handler)
	{
		this.handler = handler;
	}
	
	protected Object invokeHandler(IMethodHandler methodHandler,
			Object [] args) throws Exception
	{
		return handler.invoke(target, methodHandler, args);
	}
	
	/**
	 * Wraps an exception that might have been thrown by
	 * a proxied method.
	 * 
	 * @param e  an exception to be wrapped
	 * @return wrapped exception
	 */
	protected RuntimeException wrap(Exception e)
	{
		return new RuntimeException(e.toString());
	}
}
