/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.runtime.core.context.proxy;


import java.util.Hashtable;


public abstract class AbstractProxyClass
		implements IInvocationHandler, IProxyClass
{
	private Hashtable methodHandlers = new Hashtable();

	public Object invoke(IProxyTarget target, IMethodHandler handler,
			Object[] args) throws Exception
	{
		return handler.invoke(target, args);
	}
	
	public IMethodHandler getMethodHandler(
					String methodName, Class [] types)
	{
		IMethodHandler [] h
				= (IMethodHandler[]) methodHandlers.get(methodName);
		
		if (h != null)
		{
			for (int i = 0; i < h.length; i++)
			{
				if (h[i].matchesArgumentTypes(types))
				{
					return h[i];
				}
			}
		}
		
		throw new MethodNotFoundException(methodName, types);
	}
	
	protected void registerHandlers(IMethodHandler [] handlers)
	{
		if ((handlers != null) && (handlers.length > 0))
		{
			methodHandlers.put(handlers[0].getMethodName(), handlers);
		}
	}
	
	protected void registerHandler(IMethodHandler handler)
	{
		registerHandlers(new IMethodHandler[]{handler});
	}
}
