/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.runtime.core.context.proxy;


/**
 * A proxy object for a bean registered in a context.
 * The notion of a reference is needed to support lazy
 * initialization.
 * 
 * @author Marek
 */
public interface IBeanDelegate
{	
	/**
	 * Returns the wrapped bean.
	 * 
	 * @return wrapped bean
	 * @throws ContextException if an error occurs while
	 *         retrieving the bean
	 */
	Object getBean() throws Exception;
	
	/**
	 * Returns the class of the wrapped bean.
	 * This method is useful for determining a class
	 * of a bean that may not actually be created until
	 * {@link #getBean()} is invoked.
	 *
	 * @return class of the wrapped bean
	 */
	Class getBeanClass();
}
