/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.runtime.core.context.proxy;


import java.lang.reflect.Method;


/**
 * Provides information on a class method.
 * 
 * This is an equivalent of the {@link Method} class.
 * 
 * @author Marek
 */
public interface IMethodHandler
{
	/**
	 * 
	 * Invokes a method this object represents on the provided instance.
	 * 
	 * @param target object instance to invoke the method on
	 * @param args   method arguments
	 * @return       value returned by the method 
	 *               (null if the return type is void)
	 * @throws Exception
	 */
	Object invoke(IProxyTarget target, Object[] args) throws Exception;

	/**
	 * Returns the name of this method object
	 * 
	 * @return name of this method
	 */
	String getMethodName();

	/**
	 * Returns the types of arguments this methods accepts.
	 * 
	 * @return a sequence of types that matches the signature
	 *         of this method object (an empty array if the method
	 *         does not accept any arguments)  
	 */
	Class [] getArgumentTypes();

	/**
	 * Determines if the signature of this method object matches
	 * the provided sequence of types.
	 *
	 * @param types a sequence of types to compare the signature
	 *              of this method object to
	 *
	 * @return true if a match is found, false otherwise
	 */
	boolean matchesArgumentTypes(Class [] types);
}
