/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.generator.source;


import java.io.Writer;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import com.aurorasoftworks.signal.tools.core.context.ContextProcessingException;
import com.aurorasoftworks.signal.tools.core.context.generator.ContextGeneratorException;
import com.aurorasoftworks.signal.tools.core.context.loader.BeanInfo;
import com.aurorasoftworks.signal.tools.core.context.loader.IBeanInfo;
import com.aurorasoftworks.signal.tools.core.context.loader.IContextLoader;
import com.aurorasoftworks.signal.tools.core.context.loader.IContextLoaderListener;


public abstract class AbstractContextSourceGenerator<T>
{
	private SourceWriter writer;
	
	private T context;
	
	private ClassLoader classLoader = getClass().getClassLoader();
	
	private IBeanInfo currentBean;
	
	private Set<IBeanInfo> processedBeans = new LinkedHashSet<IBeanInfo>();
	
	protected final void generateContext(String contextLocation,
			IContextLoader<T> loader, Writer writer)
			throws ContextProcessingException
	{
		this.writer = new SourceWriter(writer);
		this.currentBean = null;
		this.processedBeans.clear();
		
		doGenerateContext(contextLocation, loader);
	}
	
	protected void doGenerateContext(String contextLocation,
			IContextLoader<T> loader)
			throws ContextProcessingException
	{
		context = loader.loadContext(contextLocation,
				getContextLoaderListener(), classLoader);
		checkBean((IBeanInfo) null);
	}
	
	protected abstract IContextLoaderListener getContextLoaderListener();

	protected SourceWriter getWriter()
	{
		return writer;
	}

	protected T getContext()
	{
		return context;
	}

	protected IBeanInfo getCurrentBean()
	{
		return currentBean;
	}
	
	protected void checkBean(String beanName, Object bean)
			throws ContextGeneratorException
	{
		checkBean(new BeanInfo(beanName, bean));
	}
	
	protected void checkBean(IBeanInfo bean) throws ContextGeneratorException
	{
		if ((currentBean == null) || (!currentBean.equals(bean)))
		{
			if (currentBean != null)
			{
				onBeanGenerationCompleted(currentBean);
			}
			if (bean != null)
			{
				currentBean = bean;
				onBeanGenerationStarted(bean);
				
				if (bean != null)
				{
					processedBeans.add(bean);
				}
			}
		}
	}

	protected void onBeanGenerationStarted(IBeanInfo bean)
	{
	}
	
	protected void onBeanGenerationCompleted(IBeanInfo bean)
			throws ContextGeneratorException
	{
	}

	public List<IBeanInfo> getProcessedBeans()
	{
		return new ArrayList<IBeanInfo>(processedBeans);
	}

	public ClassLoader getClassLoader()
	{
		return classLoader;
	}

	public void setClassLoader(ClassLoader classLoader)
	{
		this.classLoader = classLoader;
	}
}
