/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.generator.source;


import java.io.PrintWriter;
import java.io.Writer;


public class SourceWriter extends PrintWriter
{
	public static final String INDENT = "    ";
	
	private int indentationLevel = 0;

	public SourceWriter(Writer out)
	{
		super(out);
	}
	
	public int indent()
	{
		return ++indentationLevel;
	}
	
	public int unindent()
	{
		indentationLevel--;

		if (indentationLevel < 0)
		{
			throw new RuntimeException("Indentation cannot be negative");
		}
		
		return indentationLevel;
	}

	@Override
	public void write(String s)
	{
		StringBuilder buf = new StringBuilder(s);
		
		for (int i = 0; i < indentationLevel; i++)
		{
			buf.insert(0, INDENT);
		}
		
		super.write(buf.toString());
	}
}
