package com.aurorasoftworks.signal.tools.core.context.generator.source.java;


import static com.aurorasoftworks.signal.tools.core.context.generator.source.java.JavaSourceUtils.formatCast;
import static com.aurorasoftworks.signal.tools.core.context.generator.source.java.JavaSourceUtils.formatHandlerName;
import static com.aurorasoftworks.signal.tools.core.context.generator.source.java.JavaSourceUtils.formatUnwrapPrimitive;
import static com.aurorasoftworks.signal.tools.core.context.generator.source.java.JavaSourceUtils.formatWrapPrimitive;
import static com.aurorasoftworks.signal.tools.core.context.generator.source.java.JavaSourceUtils.isPrimitive;
import static com.aurorasoftworks.signal.tools.core.context.generator.source.java.JavaSourceUtils.primitiveToWrapper;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import com.aurorasoftworks.signal.tools.core.context.loader.IBeanInfo;
import com.aurorasoftworks.signal.tools.core.util.BeanUtils;



public abstract class AbstractJavaSourceProxyGenerator
{
	protected AbstractJavaSourceProxyGenerator()
	{
	}

	protected String formatInterfaces(IBeanInfo bean)
	{
		StringBuilder buf = new StringBuilder();
		List<Class<?>> interfaces
				= new ArrayList<Class<?>>(
						BeanUtils.getAllClassInterfaces(bean.getBeanClass()));
		
		for (int i = 0; i < interfaces.size(); i++)
		{
			buf.append(interfaces.get(i).getName());
			if (i < interfaces.size() - 1)
			{
				buf.append(", ");
			}
		}
		
		return buf.toString();
	}
	
	protected String formatArgArrayName(int length)
	{
		return "args" + length;
	}

	protected String formatWrapArgument(String arg, Class<?> cls)
	{
		String result;
		
		if (isPrimitive(cls))
		{
			result = formatWrapPrimitive(arg, cls);
		}
		else
		{
			result = arg;
		}
		
		return result;
	}

	protected String formatReturnValue(Method m, int index)
	{
		return formatReturnValue("invokeHandler("
				+ formatHandlerName(index) + ", "
				+ formatArgArrayName(m.getParameterTypes().length)
				+ ")",
				m.getReturnType());
	}

	protected String formatReturnValue(String expression, Class<?> type)
	{
		String result;
		
		if (void.class.equals(type))
		{
			result = expression;
		}
		else if (isPrimitive(type))
		{
			result = formatUnwrapPrimitive(
					formatCast(expression, primitiveToWrapper(type)),
					type);
		}
		else
		{
			result = formatCast(expression, type);
		}
		
		return result;
	}

	protected String formatPrimitiveReturnValue(String expression, String wrapper,
			String getter)
	{
		return "((" + wrapper + ")" + expression + ")." + getter;
	}

	protected String formatArguments(Method m)
	{
		StringBuilder buf = new StringBuilder();
		int length = m.getParameterTypes().length;
		
		for (int i = 0; i < length; i++)
		{
			buf.append(m.getParameterTypes()[i].getName());
			buf.append(' ');
			buf.append(formatArgName(i));
			if (i < length - 1)
			{
				buf.append(", ");
			}
		}
		
		return buf.toString();
	}

	protected String formatArgName(int i)
	{
		return "a" + i;
	}
}
