/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.loader;


import java.util.Set;

import com.aurorasoftworks.signal.runtime.core.context.proxy.IProxyTarget;
import com.aurorasoftworks.signal.tools.core.util.BeanUtils;



/**
 * A default implementation of the {@link IBeanInfo} interface.
 * 
 * @author Marek
 */
public class BeanInfo implements IBeanInfo
{
	private String name;

	private Object bean;
	
	private boolean lazyInit;
	
	public BeanInfo(String name, Object bean, boolean lazyInit)
	{
		this.name = name;
		this.bean = bean;
		this.lazyInit = lazyInit;
	}
	
	public BeanInfo(String name, Object bean)
	{
		this(name, bean, false);
	}

	public String getName()
	{
		return name;
	}

	public Object getBean()
	{
		return bean;
	}

	public boolean isLazyInit()
	{
		return lazyInit;
	}
	
	@Override
	public int hashCode()
	{
		return name.hashCode();
	}

	@Override
	public boolean equals(Object obj)
	{
		boolean result = false;
		
		if (obj instanceof IBeanInfo)
		{
			IBeanInfo other = (IBeanInfo) obj;
			result = name.equals(other.getName());
		}
		
		return result;
	}

	public Class<?> getBeanClass()
	{
		return getBean().getClass();
	}

	public String getPackageName()
	{
		return BeanUtils.getPackageName(getFullClassName());
	}
	
	public String getFullClassName()
	{
		return getBeanClass().getName();
	}

	public String getShortClassName()
	{
		return BeanUtils.getClassName(getFullClassName());
	}

	public boolean isProxyTarget()
	{
		boolean result = bean instanceof IProxyTarget;
		
		if (result)
		{
			/* Check if any interfaces other than IProxyTarget
			 * are implemented */
			Set<Class<?>> interfaces
					= BeanUtils.getAllClassInterfaces(getBeanClass());
			interfaces.remove(IProxyTarget.class);
			result = interfaces.size() > 0;
		}
		
		return result;
	}
}
