package com.aurorasoftworks.signal.tools.core.context.loader;


import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.List;

import com.aurorasoftworks.signal.tools.core.context.ContextProcessingException;


public class SortingContextLoaderListener
{
	static class Invocation
	{
		private Method method;
		
		private Object [] arguments;
		
		Invocation(Method method, Object[] arguments)
		{
			this.method = method;
			this.arguments = arguments;
		}

		void invoke(IContextLoaderListener listener)
				throws Exception
		{
			method.invoke(listener, arguments);
		}
	}
	
	static class InvocationList
	{
		private IBeanInfo info;

		private List<Invocation> invocations = new ArrayList<Invocation>();
		
		public InvocationList(IBeanInfo info)
		{
			this.info = info;
		}
		
		void invoke(IContextLoaderListener listener)
				throws Exception
		{
			for (Invocation invocation : invocations)
			{
				invocation.invoke(listener);
			}
		}
		
		void addInvocation(Invocation invocation)
		{
			if ("onBeanPreProcess".equals(invocation.method.getName()))
			{
				invocations.add(invocations.size() == 0 ? 0 : 1, invocation);
			}
			else
			{
				invocations.add(invocation);
			}
		}

		IBeanInfo getInfo()
		{
			return info;
		}
	}
	
	private List<InvocationList> invocationLists;
	
	private IContextLoaderListener proxy;
	
	public SortingContextLoaderListener()
	{
		invocationLists = new ArrayList<InvocationList>();
		
		proxy = (IContextLoaderListener) Proxy.newProxyInstance(
				getClass().getClassLoader(),
				new Class<?>[]{IContextLoaderListener.class},
				new InvocationHandler()
				{
					public Object invoke(Object proxy, Method method,
							Object[] args) throws Throwable
					{
						registerInvocation(method, args);
						return null;
					}
				});
	}

	protected void registerInvocation(Method method, Object[] args)
	{
		IBeanInfo info = (IBeanInfo) args[0];
		getInvocationList(info).addInvocation(new Invocation(method, args));
	}

	private InvocationList getInvocationList(IBeanInfo info)
	{
		InvocationList result = null;
		
		if (info.isLazyInit())
		{
			for (InvocationList inv : invocationLists)
			{
				if (info.equals(inv.getInfo()))
				{
					result = inv;
					break;
				}
			}
		}

		if (result == null)
		{
			result = new InvocationList(info);
			invocationLists.add(result);
		}
		
		return result;
	}

	public IContextLoaderListener getProxy()
	{
		return proxy;
	}
	
	public void invoke(IContextLoaderListener listener)
			throws ContextProcessingException
	{
		for (InvocationList inv : invocationLists)
		{
			try
			{
				inv.invoke(listener);
			}
			catch (Exception ex)
			{
				throw new ContextProcessingException(ex);
			}
		}
	}
}
