/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.loader.spring;



import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.AbstractRefreshableConfigApplicationContext;

import com.aurorasoftworks.signal.tools.core.context.ContextProcessingException;
import com.aurorasoftworks.signal.tools.core.context.loader.ContextLoaderException;
import com.aurorasoftworks.signal.tools.core.context.loader.IContextLoader;
import com.aurorasoftworks.signal.tools.core.context.loader.IContextLoaderListener;
import com.aurorasoftworks.signal.tools.core.context.loader.SortingContextLoaderListener;


public abstract class AbstractContextLoader
		implements IContextLoader<ApplicationContext>
{
	private static final Log LOGGER
			= LogFactory.getLog(AbstractContextLoader.class);
	
	public ApplicationContext loadContext(String contextLocation,
			IContextLoaderListener listener)
			throws ContextProcessingException
	{
		return loadContext(contextLocation, listener,
				getClass().getClassLoader());
	}
	
	public ApplicationContext loadContext(String contextLocation,
			IContextLoaderListener listener, ClassLoader classLoader)
			throws ContextProcessingException
	{
		LOGGER.debug("Loading context: " + contextLocation);
		
		SortingContextLoaderListener orderedListener
				= new SortingContextLoaderListener();
		AbstractRefreshableConfigApplicationContext ctx
				= createContext(orderedListener.getProxy());
		ctx.setConfigLocation(contextLocation);
		ctx.setClassLoader(classLoader);
		ctx.refresh();
		
		postProcessContext(ctx);
		
		orderedListener.invoke(listener);
				
		return ctx;
	}

	protected void postProcessContext(
			AbstractRefreshableConfigApplicationContext ctx)
			throws ContextLoaderException
	{
		processLazyInitBeans(ctx);
	}

	protected void processLazyInitBeans(
			AbstractRefreshableConfigApplicationContext ctx)
			throws ContextLoaderException
	{
		BeanFactory factory = ctx.getBeanFactory();
		
		if (factory instanceof ConfigurableListableBeanFactory)
		{
			ConfigurableListableBeanFactory confFactory
					= (ConfigurableListableBeanFactory) factory;
			
			for (String name : confFactory.getBeanDefinitionNames())
			{
				BeanDefinition def = confFactory.getBeanDefinition(name);
				
				if (def.isLazyInit())
				{
					ctx.getBean(name);
				}
			}
		}
	}

	protected abstract AbstractRefreshableConfigApplicationContext createContext(
			IContextLoaderListener listener);
}
