/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.loader.spring;


import java.util.LinkedHashMap;
import java.util.Map;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.aurorasoftworks.signal.tools.core.context.loader.BeanReference;
import com.aurorasoftworks.signal.tools.core.context.loader.IBeanReference;


public class AutowiringState implements IAutowiringState
{
	private static final Log LOGGER
		= LogFactory.getLog(AutowiringState.class);
	
	private Map<String, IBeanReference> references
			= new LinkedHashMap<String, IBeanReference>();
	
	public void saveBeanReference(String name, Object object)
	{
		saveBeanReference(new BeanReference(name, object));
	}

	public void saveBeanReference(IBeanReference ref)
	{
		LOGGER.debug("Saving a bean reference: " + ref.toString());
		references.put(ref.getName(), ref);
	}

	public IBeanReference getBeanReference(Object object)
	{
		IBeanReference ref = findBeanReference(object);
		
		if (ref == null)
		{
			throw new RuntimeException("Reference not found for " + object);
		}
		
		return ref;
	}
	
	public IBeanReference getBeanReference(String name)
	{
		IBeanReference ref = references.get(name);
		
		if (ref == null)
		{
			throw new RuntimeException("Reference not found for " + name);
		}
		
		return ref;
	}

	protected IBeanReference findBeanReference(Object object)
	{
		IBeanReference result = null;
		
		for (Map.Entry<String, IBeanReference> e : references.entrySet())
		{
			if (e.getValue().getObject() == object)
			{
				result = e.getValue();
				break;
			}
		}
		
		return result;
	}

	public boolean isBeanReference(Object object)
	{
		return findBeanReference(object) != null;
	}
	
	public boolean isBeanReference(String name)
	{
		return references.containsKey(name);
	}
}
