/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.loader.spring;


import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.util.Hashtable;
import java.util.Map;


public class HashtablePropertyEditor extends PropertyEditorSupport
{
	private static final PropertyEditor INSTANCE
			= new HashtablePropertyEditor();
	
	public static PropertyEditor getInstance()
	{
		return INSTANCE;
	}
	
	@Override
	public void setValue(Object value)
	{
		Object newValue;
		
		if (value instanceof Map<?, ?>)
		{
			Hashtable<Object, Object> ht = new Hashtable<Object, Object>();
			Map <?, ?> map = (Map<?, ?>) value;
			
			for (Map.Entry<?, ?> e : map.entrySet())
			{
				ht.put(e.getKey(), e.getValue());
			}
			
			newValue = ht;
		}
		else
		{
			newValue = value;
		}
		
		super.setValue(newValue);
	}
}
