/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.loader.spring;


import java.beans.PropertyDescriptor;
import java.beans.PropertyEditor;
import java.util.Iterator;
import java.util.Map;


import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeansException;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.PropertyValues;
import org.springframework.beans.TypeMismatchException;
import org.springframework.beans.factory.BeanCreationException;
import org.springframework.core.MethodParameter;

import com.aurorasoftworks.signal.tools.core.context.generator.ContextGeneratorException;
import com.aurorasoftworks.signal.tools.core.context.loader.BeanInfo;
import com.aurorasoftworks.signal.tools.core.context.loader.IContextLoaderListener;
import com.aurorasoftworks.signal.tools.core.util.BeanUtils;


public class LoggingBeanWrapper implements BeanWrapper
{
	private BeanWrapper wrapper;
	
	private String beanName;
	
	private boolean isLazyInit;
	
	private IContextLoaderListener listener;
	
	private IBeanReferenceResolver referenceResolver;
	
	public LoggingBeanWrapper(BeanWrapper wrapper,
			String beanName, boolean isLazyInit,
			IContextLoaderListener listener,
			IBeanReferenceResolver referenceResolver)
	{
		this.wrapper = wrapper;
		this.beanName = beanName;
		this.isLazyInit = isLazyInit;
		this.listener = listener;
		this.referenceResolver = referenceResolver;
	}

	public Object convertIfNecessary(Object value, Class requiredType,
			MethodParameter methodParam) throws TypeMismatchException
	{
		return wrapper.convertIfNecessary(value, requiredType, methodParam);
	}

	public Object convertIfNecessary(Object value, Class requiredType)
			throws TypeMismatchException
	{
		return wrapper.convertIfNecessary(value, requiredType);
	}

	public PropertyEditor findCustomEditor(Class requiredType,
			String propertyPath)
	{
		return wrapper.findCustomEditor(requiredType, propertyPath);
	}

	public PropertyDescriptor getPropertyDescriptor(String propertyName)
			throws BeansException
	{
		return wrapper.getPropertyDescriptor(propertyName);
	}

	public PropertyDescriptor[] getPropertyDescriptors()
	{
		return wrapper.getPropertyDescriptors();
	}

	public Class getPropertyType(String propertyName) throws BeansException
	{
		return wrapper.getPropertyType(propertyName);
	}

	public Object getPropertyValue(String propertyName) throws BeansException
	{
		return wrapper.getPropertyValue(propertyName);
	}

	public Class getWrappedClass()
	{
		return wrapper.getWrappedClass();
	}

	public Object getWrappedInstance()
	{
		return wrapper.getWrappedInstance();
	}

	public boolean isExtractOldValueForEditor()
	{
		return wrapper.isExtractOldValueForEditor();
	}

	public boolean isReadableProperty(String propertyName)
	{
		return wrapper.isReadableProperty(propertyName);
	}

	public boolean isWritableProperty(String propertyName)
	{
		return wrapper.isWritableProperty(propertyName);
	}

	public void registerCustomEditor(Class requiredType,
			PropertyEditor propertyEditor)
	{
		wrapper.registerCustomEditor(requiredType, propertyEditor);
	}

	public void registerCustomEditor(Class requiredType, String propertyPath,
			PropertyEditor propertyEditor)
	{
		wrapper.registerCustomEditor(requiredType, propertyPath,
				propertyEditor);
	}

	public void setExtractOldValueForEditor(boolean extractOldValueForEditor)
	{
		wrapper.setExtractOldValueForEditor(extractOldValueForEditor);
	}

	public void setPropertyValue(PropertyValue pv) throws BeansException
	{
		wrapper.setPropertyValue(pv);
		logPropertyValue(pv);
	}

	public void setPropertyValue(String propertyName, Object value)
			throws BeansException
	{
		wrapper.setPropertyValue(propertyName, value);
		logPropertyValue(propertyName, value);
	}

	public void setPropertyValues(Map map) throws BeansException
	{
		wrapper.setPropertyValues(map);
		logPropertyValue(map);
	}

	public void setPropertyValues(PropertyValues pvs, boolean ignoreUnknown,
			boolean ignoreInvalid) throws BeansException
	{
		wrapper.setPropertyValues(pvs, ignoreUnknown, ignoreInvalid);
		logPropertyValue(pvs);
	}

	public void setPropertyValues(PropertyValues pvs, boolean ignoreUnknown)
			throws BeansException
	{
		wrapper.setPropertyValues(pvs, ignoreUnknown);
		logPropertyValue(pvs);
	}

	public void setPropertyValues(PropertyValues pvs) throws BeansException
	{
		wrapper.setPropertyValues(pvs);
		logPropertyValue(pvs);
	}

	public void setWrappedInstance(Object obj)
	{
		wrapper.setWrappedInstance(obj);
	}
	
	protected void logPropertyValue(PropertyValues pvs)
	{
		for (PropertyValue pv : pvs.getPropertyValues())
		{
			logPropertyValue(pv);
		}
	}
	
	protected void logPropertyValue(Map properties)
	{
		for (Iterator<Map.Entry> it = properties.entrySet().iterator(); it.hasNext();)
		{
			Map.Entry entry = it.next();
			logPropertyValue((String) entry.getKey(), entry.getValue());
		}
	}
	
	protected void logPropertyValue(PropertyValue pv)
	{
		logPropertyValue(pv.getName(),
				pv.isConverted()
				? pv.getConvertedValue()
				: pv.getValue());
	}

	protected void logPropertyValue(String name, Object value)
	{
		Object resolvedValue = BeanUtils.isPrimitive(value)
				? value : referenceResolver.getBeanReference(value);
		try
		{
			listener.onBeanPropertySet(new BeanInfo(beanName,
					getWrappedInstance(), isLazyInit), name, resolvedValue);
		}
		catch (ContextGeneratorException e)
		{
			throw new BeanCreationException(
					"Failed to set property: " + name, e);
		}
	}
}
