package com.aurorasoftworks.signal.tools.core.context.generator.source.java;


import static org.testng.AssertJUnit.assertTrue;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;


import org.apache.commons.jci.compilers.CompilationResult;
import org.apache.commons.jci.compilers.JavaCompiler;
import org.apache.commons.jci.compilers.JavaCompilerFactory;
import org.apache.commons.jci.readers.FileResourceReader;
import org.apache.commons.jci.stores.FileResourceStore;
import org.springframework.context.ApplicationContext;

import com.aurorasoftworks.signal.runtime.core.context.Context;
import com.aurorasoftworks.signal.runtime.core.context.IContext;
import com.aurorasoftworks.signal.tools.core.context.ContextProcessingException;
import com.aurorasoftworks.signal.tools.core.context.generator.source.java.JavaSourceContextGenerator;
import com.aurorasoftworks.signal.tools.core.context.loader.IContextLoader;
import com.aurorasoftworks.signal.tools.core.context.loader.spring.ClassPathContextLoader;


public class AbstractSourceContextGeneratorTest
{

	public AbstractSourceContextGeneratorTest()
	{
		super();
	}

	protected String getContextClassName()
	{
		return "com.aurorasoftworks.signal.TestContext";
	}

	protected void purge(File dir)
	{
		for (File f : dir.listFiles())
		{
			if (f.isFile())
			{
				f.delete();
			}
			else
			{
				purge(f);
			}
		}	
	}

	protected String [] findSourceFileNames(File dir)
	{
		List<File> files = findSourceFiles(dir);
		String [] names = new String[files.size()];
		
		for (int i = 0; i < names.length; i++)
		{
			names[i] = files.get(i).getAbsolutePath().substring(
					dir.getAbsolutePath().length());
		}
		
		return names;
	}

	protected List<File> findSourceFiles(File dir)
	{
		List<File> results = new ArrayList<File>();
		
		for (File f : dir.listFiles())
		{
			if (f.isDirectory())
			{
				results.addAll(findSourceFiles(f));
			}
			else if (f.getName().endsWith(".java"))
			{
				results.add(f);
			}
		}
		
		return results;
	}

	protected IContext generate(String contextLocation) throws Exception
	{
		IContextLoader<ApplicationContext> loader
				= ClassPathContextLoader.getInstance();
		File rootDir = getGeneratedClassDirectory();
		if (rootDir.exists())
		{
			purge(rootDir);
		}
		rootDir.mkdirs();
		File file = new File(getContextClassName().replace(
				'.', File.separatorChar) + ".java");
		
		generate(contextLocation, loader, rootDir);
		compile(rootDir, file);
		return load((Class<?>) loadGeneratedClass(getContextClassName()));
	}

	protected Class<?> loadGeneratedClass(String className)
			throws ClassNotFoundException, MalformedURLException
	{
		return getGeneratedClassLoader().loadClass(className);
	}

	protected URLClassLoader getGeneratedClassLoader() throws MalformedURLException
	{
		return new URLClassLoader(new URL[]{
				getGeneratedClassDirectory().toURI().toURL()},
				this.getClass().getClassLoader());
	}

	protected File getGeneratedClassDirectory()
	{
		return new File("target" + File.separator + "tmp");
	}

	protected TestProxyFactory createProxyFactory() throws MalformedURLException
	{
		return new TestProxyFactory(getGeneratedClassLoader());
	}

	protected IContext load(Class<?> cls) throws MalformedURLException,
			ClassNotFoundException, InstantiationException,
			IllegalAccessException
	{
		assertTrue(IContext.class.isAssignableFrom(cls));
		Context.setDefaultProxyFactory(createProxyFactory());
		
		return (IContext) cls.newInstance();
	}

	protected void compile(File baseDir, File file) throws Exception
	{
		FileReader reader = new FileReader(new File(baseDir, file.toString()));
		try
		{
			JavaCompiler compiler = new JavaCompilerFactory()
					.createCompiler("janino");
			CompilationResult result = compiler.compile(
					findSourceFileNames(baseDir),
					new FileResourceReader(baseDir),
					new FileResourceStore(baseDir));
			if ((result.getErrors() != null)
					&& (result.getErrors().length > 0))
			{
				throw new Exception("Compilation failed, file: "
						+ file + ", dir: " + baseDir + ", "
						+ result.getErrors()[0]);
			}
		}
		finally
		{
			reader.close();
		}
	}

	protected void generate(String contextLocation,
			IContextLoader<ApplicationContext> loader, File outputDir)
			throws IOException, ContextProcessingException
	{
		new JavaSourceContextGenerator<ApplicationContext>().generateContext(
				contextLocation,
				getContextClassName(),
				loader, outputDir);
	}
}
