/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.aurorasoftworks.signal.tools.core.context.generator.source.java;


import java.net.MalformedURLException;


import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.testng.annotations.Test;

import com.aurorasoftworks.signal.runtime.core.context.IContext;


@Test
public class JavaDefaultPackageContextGeneratorTest
		extends	JavaSourceContextGeneratorTest
{
	protected static final String DEFAULT_CONTEXT_LOCATION
			= "/com/aurorasoftworks/signal/tools/core/context/loader/spring/primitive-properties.xml";

	@Override
	protected String getContextClassName()
	{
		return "TestContext";
	}

	public void testPerformance() throws Exception
	{
		/* Warmup */
		loadXmlContext(100);
		loadContextClass(100);
		
		long t0 = System.currentTimeMillis();
		loadXmlContext(200);
		long t = System.currentTimeMillis() - t0;
		System.out.println("XML loader: " + t);
		
		t0 = System.currentTimeMillis();
		loadContextClass(200);
		t = System.currentTimeMillis() - t0;
		System.out.println("Class loader: " + t);
	}
	
	protected IContext load(String className)
			throws Exception
	{
		return (IContext) getGeneratedClassLoader().loadClass(className)
				.newInstance();
	}

	protected void loadXmlContext(int count)
	{
		for (int i = 0; i < count; i++)
		{
			new ClassPathXmlApplicationContext(DEFAULT_CONTEXT_LOCATION)
					.getBean("props");
		}
	}
	
	protected void loadContextClass(int count) throws Exception
	{
		generate(DEFAULT_CONTEXT_LOCATION);
		
		for (int i = 0; i < count; i++)
		{
			load(getContextClassName()).getBean("props");
		}
	}
}
