package com.aurorasoftworks.signal.tools.core.context.generator.source.java;


import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;


import org.testng.annotations.Test;

import com.aurorasoftworks.signal.runtime.core.context.proxy.AbstractMethodInterceptor;
import com.aurorasoftworks.signal.runtime.core.context.proxy.IMethodHandler;
import com.aurorasoftworks.signal.runtime.core.context.proxy.IMethodInterceptor;
import com.aurorasoftworks.signal.runtime.core.context.proxy.IProxyFactory;
import com.aurorasoftworks.signal.runtime.core.context.proxy.IProxyTarget;
import com.aurorasoftworks.signal.tools.core.context.loader.IProxyTargetSibling;
import com.aurorasoftworks.signal.tools.core.context.loader.ProxyTarget;


@Test
public class JavaSourceContextGeneratorPerfTest extends
		AbstractSourceContextGeneratorTest
{
	public void testProxyTargetWithInterceptors() throws Exception
	{
		generate("/com/aurorasoftworks/signal/tools/core/context/loader/spring/proxy-target.xml");

		IProxyFactory proxyFactory = createProxyFactory();
		ProxyTarget target = new ProxyTarget();
		IProxyTargetSibling proxy = (IProxyTargetSibling) proxyFactory
				.createProxy(target,
						new IMethodInterceptor[]{new AbstractMethodInterceptor()
						{
							public Object invoke(IProxyTarget target,
									IMethodHandler handler, Object[] args)
									throws Exception
							{
								return ((String )nextHandler.invoke(
										target, handler, args)).toUpperCase();
							}
						}});

		/* Warmup */
		execute(target, 10000);
		execute(proxy, 10000);
		
		long targetTime = execute(target, 1000000);
		long proxyTime = execute(proxy, 1000000);
		printDelta("Overhead", targetTime, proxyTime);
		
		targetTime = executeParallel(target, 1000000, 2);
		proxyTime = executeParallel(proxy, 1000000, 2);
		printDelta("Parallel overhead", targetTime, proxyTime);
	}

	protected void printDelta(String label, long targetTime, long proxyTime)
	{
		System.out.println(label + ": "
				+ NumberFormat.getPercentInstance().format(
				(proxyTime - targetTime) / (double) targetTime) + " of "
				+ targetTime + " ms");
	}

	protected long execute(IProxyTargetSibling target, int count)
	{
		long t0 = System.currentTimeMillis();
		
		for (int i = 0; i < count; i++)
		{
			target.fourth(
				1, 2l, (short) 3, (byte) 4, 'a', 1.5f, 2.5, "xyz");
		}
		
		return System.currentTimeMillis() - t0;
	}
	
	protected long executeParallel(final IProxyTargetSibling target,
			final int count, final int threadCount)
			throws InterruptedException, ExecutionException
	{
		ExecutorService exec = Executors.newFixedThreadPool(threadCount);
		
		Callable<Object> call = new Callable<Object>()
		{
			public Object call() throws Exception
			{
				for (int i = 0; i < count / threadCount; i++)
				{
					target.fourth(
							1, 2l, (short) 3, (byte) 4, 'a', 1.5f, 2.5, "xyz");
				}
				return null;
			}
		};
		
		List<Future<Object>> future = new ArrayList<Future<Object>>(); 
		long t0 = System.currentTimeMillis();
		
		for (int i = 0; i < threadCount; i++)
		{
			future.add(exec.submit(call));
		}
		
		for (Future<Object> f : future)
		{
			f.get();
		}
		
		return System.currentTimeMillis() - t0;
	}
}
